#ifndef controlpointtypetools_h_included
#define controlpointtypetools_h_included
/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x Tools that change control point type
 *--------------------------------------------------------------------------
 * @histroy: 02.10.2003. Created
 */

#include <zmodeler.h>
#include <helpers/layout.h>
#include <core/layout/IMultyToolLayout.h>

namespace core
{
  namespace tools
  {

    //-------------------------------------------------------------------------
    // @Name        : class CControlPointType
    // @Purpose     : Implements spline control point type tool
    // @Description : Allows changing spline's control point type
    //-------------------------------------------------------------------------
    class CControlPointType : public CContextMenuToolEx,
                              public core::layout::IMultyToolLayout,
                              public core::CStatus
    {
    public:
      INTERFACE_DECLAREMAP(CControlPointType)
      CControlPointType();
      virtual ~CControlPointType();

    //---------------------------------------
    //    IStatus
    //---------------------------------------
    public:
      virtual ZRESULT getStatus(DWORD& nBits);
      virtual ZRESULT checkStatus(DWORD nBit);

    public:
    //-----------------------
    // IMenuLayout:
    //-----------------------
      virtual ZRESULT getBranch(ZStringSeq& branch) const;
      virtual ZRESULT getItemKind(DWORD& kind) const;

    public:
    //-----------------------
    // IMultyToolLayout:
    //-----------------------
      virtual ZRESULT setCurrentTool(long nItem);

    public:
    //-----------------------
    // overloads:
    //-----------------------
      //INamed
      virtual ZRESULT getTitle(ZString& strTitle);
      virtual ZRESULT setTitle(ZString strTitle);

    public:
      //---------------------------------------
      //    ITool
      //---------------------------------------
      ZRESULT apply(core::IProcParams* pParams = NULL, DWORD* pRetVal = NULL);
      virtual ZRESULT queryLayout(scene::INode* pNode, core::eLevel level, core::ui::IViewport* pViewport);

    protected:
      virtual ZRESULT alterPoint();
      enum eCurrTool
      {
        CORNER        = 0,
        BEZIER        = 1,
        BEZIERCORNER  = 2,
        AUTODETAIL    = 3,
      };
      eCurrTool                     m_CurrTool;

      ZString                       m_title[4];
      scene::eControlPoint          m_type;//curr point type
      bool                          m_bAuto;
      ZStdPtr<scene::INode>         m_pNode;
      ZStdPtr<scene::ISpline>       m_pSpline;
      ZStdPtr<scene::IControlPoint> m_pThis;
      ZStdPtr<scene::IControlPoint> m_pPrev;
      ZStdPtr<scene::IControlPoint> m_pNext;
    };//class

  };//tools
};//core

#endif //controlpointtypetools_h_included

