/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x Create\Spline\Helix tool implementation
 *--------------------------------------------------------------------------
 * @histroy: 24.08.2005. Created
 */
#include "Spline.h"
#include "SplineNode.h"
#include "CreateSpline.h"
#include <zmsharedclasses.h>
#include <helpers/events.h>
#include <helpers/services.h>
#include <helpers/splines.h>
#include <helpers/viewportDC.h>
#include <core/ui/ISceneViewport.h>
#include <scene/ITransformable.h>
#include <helpers/undo.h>


namespace core
{
namespace tools
{

BEGIN_MESSAGE_MAP(CHelixDialog, CChildDialog)
  ON_COMMAND(IDC_SMOOTH, OnSmooth)
END_MESSAGE_MAP()

BOOL CHelixDialog::OnInitDialog()
{
  // sizing stuff
  ZPtr<services::IDialogSizingService>  pDSS;
  if (ZRESULT_OK == services::getDialogSizingService(&pDSS))
  {
    pDSS->setDialog(this, ZRect(0, 0, 50, 20), true, false);
    pDSS->addControl(IDC_VERTSTEXT, DSS_HORIZONTAL(DSS_23_SCALE(DSS_SIZE)));
    pDSS->addControl(IDC_VERTICES,  DSS_HORIZONTAL(DSS_3_SCALE(DSS_SIZE) | DSS_23_SCALE(DSS_MOVE)));
    pDSS->addControl(IDC_SEGTEXT,   DSS_HORIZONTAL(DSS_23_SCALE(DSS_SIZE)));
    pDSS->addControl(IDC_SEGMENTS,  DSS_HORIZONTAL(DSS_3_SCALE(DSS_SIZE) | DSS_23_SCALE(DSS_MOVE)));
    pDSS->addControl(IDC_STEPSTEXT, DSS_HORIZONTAL(DSS_23_SCALE(DSS_SIZE)));
    pDSS->addControl(IDC_STEPS,     DSS_HORIZONTAL(DSS_3_SCALE(DSS_SIZE) | DSS_23_SCALE(DSS_MOVE)));
    pDSS->addControl(IDC_SMOOTH,    DSS_HORIZONTAL(DSS_SIZE));
  }
  createSharedClass(_T(ZM_CLASS_SPIN_CONTROL), IID_(core::ui::controls::ISpinControl), (void**)&m_pSegments);
  if (m_pSegments)
  {
    m_pSegments->embed(GetDlgItem(IDC_SEGMENTS));
    m_pSegments->setRange(0, 100);
    m_pSegments->setValue(8);
  }
  createSharedClass(_T(ZM_CLASS_SPIN_CONTROL), IID_(core::ui::controls::ISpinControl), (void**)&m_pVertices);
  if (m_pVertices)
  {
    m_pVertices->embed(GetDlgItem(IDC_VERTICES));
    m_pVertices->setRange(4, 128);
    m_pVertices->setValue(4);
  }
  createSharedClass(_T(ZM_CLASS_SPIN_CONTROL), IID_(core::ui::controls::ISpinControl), (void**)&m_pSteps);
  if (m_pSteps)
  {
    m_pSteps->embed(GetDlgItem(IDC_STEPS));
    m_pSteps->setRange(1, 1000);
    m_pSteps->setValue(3);
  }
  OnSmooth();
  return CChildDialog::OnInitDialog();
}

long CHelixDialog::getSegments()
{
  long n = 4;
  if (m_pSegments)
    m_pSegments->getValue(n);
  return n;
}

long CHelixDialog::getVertices()
{
  long n = 4;
  if (m_pVertices)
    m_pVertices->getValue(n);
  return n;
}

long CHelixDialog::getSteps()
{
  long n = 3;
  if (m_pSteps)
    m_pSteps->getValue(n);
  return n;
}

bool CHelixDialog::isSmooth()
{
  return 1 == ((CButton*)GetDlgItem(IDC_SMOOTH))->GetCheck();
}

void CHelixDialog::OnSmooth()
{
  bool bEnable = isSmooth();
  //GetDlgItem(IDC_SEGMENTS)->EnableWindow(bEnable);
  if (m_pSegments)
    m_pSegments->modifyStyle(bEnable ? ZM_UICONTROL_ENABLED : 0, bEnable ? 0 : ZM_UICONTROL_ENABLED);
}

//
// the tool itself
//

INTERFACE_BEGINMAP(CHelixSplineTool)
  INTERFACE_CHAIN(CCommandBarToolEx)
  INTERFACE_ENTRY(core::tools::IInteractiveTool)
INTERFACE_ENDMAP()

CHelixSplineTool::CHelixSplineTool() : CCommandBarToolEx(core::ui::CURSOR_CREATE)
{
  m_strName = _T("core::tools::CHelixSplineTool");
  m_strTitle = _T("Helix");
  m_seqBranch.add(_T("Spline..."));
  m_seqDesc.add(_T("Creates helix spline."));
  m_seqDesc.add(_T("Click and drag to draw the circle of helix spline. Expand options box for additional settings."));

  CHelixDialog::CreateInstance(&m_pDlg);
  m_pDlg->setDialogTitle(_T("Options"));
  m_pDialogs->addLast(m_pDlg);
}

CHelixSplineTool::~CHelixSplineTool()
{
}


//-------------------------------------------------------
// @name : CHelixSplineTool::requires
// Tool requirements
// @param DWORD& dwRequires              : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CHelixSplineTool::requires(DWORD& dwRequires) const
{
  dwRequires = TOOL_REQUIRES_MOUSEL | TOOL_REQUIRES_LEV_NODE;
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CHelixSplineTool::getGroup
// Tool qualifying branch
// @param core::layout::tCommandBarTab& group : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CHelixSplineTool::getGroup(core::layout::tCommandBarTab& group) const
{
  group = core::layout::LAYOUT_CB_TAB_CREATE;
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CHelixSplineTool::apply
// The application routine
// @param core::IProcParams* pParams     : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CHelixSplineTool::apply(core::IProcParams* pParams, DWORD* pRetVal)
{
  //prepare
  resetError();
  ZPtr<core::ui::IViewport>       pVPort;
  ZPtr<core::ui::ISceneViewport>  pSVPort;
  pParams->getViewport(&pVPort);
  if (!pVPort)
    return REPORT_ERROR(NULL, _T("No Viewport for CHelixSplineTool::apply."));
  if (ZRESULT_OK != pVPort->QueryInterface(IID_(core::ui::ISceneViewport), (void**)&pSVPort))
    return REPORT_ERROR(NULL, _T("Not a ISceneViewport for CHelixSplineTool::apply."));
  ZVector vDir, vUp, vRight;
  pSVPort->getViewDirection(vDir);
  pSVPort->getUpDirection(vUp);
  vDir.normalize();
  vUp.normalize();
  vRight = vDir*vUp;
  ZVector vRadius = m_ptLastPoint - m_ptOrigin;
  ZVector vHeight = vDir*vRadius.length();
  float rX = fabs(vRadius^vRight);
  float rY = fabs(vRadius^vUp);
  if (m_bCircle)
    if (rX < rY)
      rX = rY;
    else
      rY = rX;

  //
  // create helix spline
  //
  ZPtr<scene::ISpline>      pSpline;
  ZPtr<scene::ISplineNode>  pSplineNode;
  scene::createSplineNode(&pSplineNode);

  scene::CSpline::CreateInstance((void**)&pSpline);
  pSplineNode->setSpline(pSpline);
  ZPtr<scene::ITransformable> pTrans;
  if (ZRESULT_OK == pSplineNode->QueryInterface(IID_(scene::ITransformable), (void**)&pTrans))
  {
    ZMatrix mat;
    mat.setTranslation(m_ptOrigin);
    pTrans->setTransform(mat);
  }

  //create common undo
  core::undo::undoNodeOperation(_T("Create Helix"), core::undo::UNDO_ADD_NODE, pParams, pSplineNode);

  //
  // add node to the scene:
  //
  ZPtr<scene::IScene>         pScene;
  if (ZFAILED(g_pZModeler->getScene(&pScene)))
    return ZRESULT_FAIL;
  pScene->attachNode(pSplineNode, NULL);

  //
  // create spline control points
  //
  long num = m_pDlg->getVertices();
  long detail = m_pDlg->getSegments();
  long nSteps = m_pDlg->getSteps();
  bool bSmooth = m_pDlg->isSmooth();
  ZVector vZero;

  float rad = G_2PI/(float)num;
  float magic = tan(rad*0.3333f);
  float fPointStep = vHeight.length()/(float)num;
  float fManipStep = fPointStep*0.3333f;
  for (int j = 0; j < nSteps; j++)
  {
    for (int i = 0; i < num + (j == nSteps-1 ? 1:0); i++)
    {
      ZPtr<scene::IControlPoint>  pControl;
      scene::CControlPoint::CreateInstance((void**)&pControl);
      pControl->setControlPosition(
          vRight*rX*cosf(rad*i) + 
          vUp*rY*sinf(rad*i) + 
          vHeight*((float)(fPointStep*(j*num+i))));
      pControl->setControlVector(scene::SCV_U0, vZero);
      pControl->setControlVector(scene::SCV_U1, vZero);
      pControl->setType(bSmooth ? scene::SCP_BEZIER : scene::SCP_CORNER);
      pControl->setDetalization(detail);
      if (bSmooth)
      {
        //set up control vectors
        ZVector vU = -vRight*rX*sinf(rad*i)*magic + 
                      vUp*rY*cosf(rad*i)*magic + 
                      vHeight*fManipStep;
        pControl->setControlVector(scene::SCV_U0, vU);
      }

      pSpline->addLast(pControl);
    }
  }
  pSpline->setClosed(false);

  pSplineNode->changed(NC_VISUAL | NC_GEOMETRY | NC_TRANSFORM);
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CHelixSplineTool::mouseDown
// A-la starting creation
// @param core::IProcParams* pProcParams : 
// @param DWORD nFlags                   : 
// @param DWORD& nRetFlags               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CHelixSplineTool::mouseDown(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags)
{
  resetError();
  if ((0 == (nFlags & MK_LBUTTON)) || NULL == pProcParams)
    return ZRESULT_FALSE;

  pProcParams->getCursorLocation(m_ptCursorFrom);
  m_ptCursorTo = m_ptCursorFrom;
  ZLine3D line;
  pProcParams->getInputLine(line);
  m_ptOrigin = m_ptLastPoint = line[1];
  return ZRESULT_OK;//captured event
}


//-------------------------------------------------------
// @name : CHelixSplineTool::mouseMove
// Dragging the mouse
// @param core::IProcParams* pProcParams : 
// @param ZPoint ptOffset                : 
// @param DWORD nFlags                   : 
// @param DWORD& nRetFlags               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CHelixSplineTool::mouseMove(core::IProcParams* pProcParams, ZPoint ptOffset, DWORD nFlags, DWORD& nRetFlags)
{
  if (NULL == pProcParams || (0 == (nFlags & MK_LBUTTON)))
    return ZRESULT_FALSE;
  ZLine3D line;
  pProcParams->getInputLine(line);
  m_ptLastPoint = line[1];

  m_bCircle = 0 != (nFlags & MK_SHIFT);
  //
  // draw
  //
  CViewportDC dc(pProcParams);
  CPen pen;
  pen.CreatePen(PS_DOT, 1, RGB(0, 0, 255));
  dc->SetROP2(R2_XORPEN);
  dc->SelectStockObject(NULL_BRUSH);
  CPen* pOldPen = dc->SelectObject(&pen);
  //erase old
  ZPoint pt1(m_ptCursorFrom + (m_ptCursorTo-m_ptCursorFrom));
  ZPoint pt2(m_ptCursorFrom - (m_ptCursorTo-m_ptCursorFrom));
  dc->Ellipse(pt1.x(), pt1.y(), pt2.x(), pt2.y());

  pProcParams->getCursorLocation(m_ptCursorTo);
  //draw new
  if (m_bCircle)
  {
    int r = max(abs(m_ptCursorTo.x()-m_ptCursorFrom.x()), abs(m_ptCursorTo.y()-m_ptCursorFrom.y()));
    //float r = m_ptCursorFrom.distanceF(m_ptCursorTo);
    m_ptCursorTo.x(m_ptCursorFrom.x()+r);
    m_ptCursorTo.y(m_ptCursorFrom.y()+r);
    //:FIXME: alter m_ptLastPoint;
  }
  ZPoint pt3(m_ptCursorFrom + (m_ptCursorTo-m_ptCursorFrom));
  ZPoint pt4(m_ptCursorFrom - (m_ptCursorTo-m_ptCursorFrom));
  dc->Ellipse(pt3.x(), pt3.y(), pt4.x(), pt4.y());
  dc->SelectObject(pOldPen);
  return ZRESULT_OK;//captured event
}


//-------------------------------------------------------
// @name : CHelixSplineTool::mouseUp
// Create the light!
// @param core::IProcParams* pProcParams : 
// @param DWORD nFlags                   : 
// @param DWORD& nRetFlags               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CHelixSplineTool::mouseUp(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags)
{
  if ((0 == (nFlags & MK_LBUTTON)) || NULL == pProcParams)
    return ZRESULT_FALSE;
  nRetFlags |= ZRESULT_REDRAW_ALL;
  return apply(pProcParams, &nRetFlags);
}

ZRESULT CHelixSplineTool::mouseWheel(core::IProcParams* pProcParams, DWORD nFlags, short zDelta, DWORD& nRetFlags)
{
  return ZRESULT_FALSE;//not our case
}


};//tools
};//core

