/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                   Copyright (c) 2002 Zanoza Software.                   *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x SplineNode shared class
 *--------------------------------------------------------------------------
 * @histroy: 17.09.2003. Created
 */
#ifndef splinenode_class_h_included
#define splinenode_class_h_included

#include <scene/ISplineNode.h>
#include <scene/IManipulateable.h>
#include <helpers/nodes.h>
#include <helpers/status.h>
#include <helpers/serializeable.h>
#include <helpers/userDefined.h>

namespace scene
{
  //helper
  extern bool getSpline(scene::INode* pNode, ISpline** ppSpline);
  // manipulators
  class CPositionManipulator : public CNodeSpacePositionManipulator
  {
    friend class CSplineNode;
    public:
      INTERFACE_OVERLOAD_INSTANCE_CREATOR(CPositionManipulator)

      CPositionManipulator();
      virtual ~CPositionManipulator();
    public:
      virtual void getPosition(ZPoint3D& pt);
      virtual void setPosition(ZPoint3D& pt);

      //overloads
    public:
      virtual DWORD getStatus() const 
        { return m_pControl->getStatus(); }
      virtual DWORD& status()
        { return m_pControl->status(); }
      virtual DWORD  setStatus(const DWORD dwStatus)
        { return m_pControl->setStatus(dwStatus); }
      virtual DWORD  changeStatus(const DWORD dwAdd, const DWORD dwRemove)
        { return m_pControl->changeStatus(dwAdd, dwRemove); }
      virtual bool   checkStatus(const DWORD dwStatus)
        { return m_pControl->checkStatus(dwStatus); }

      void init(scene::INode* pNode, IControlPoint* pControl);
    protected:
      ZPtr<IControlPoint>     m_pControl;
  };

  class CDirectionManipulator : public CNodeSpacePositionManipulator
  {
    friend class CSplineNode;
    public:
      INTERFACE_OVERLOAD_INSTANCE_CREATOR(CDirectionManipulator)

      CDirectionManipulator();
      virtual ~CDirectionManipulator();
    public:
      virtual void getPosition(ZPoint3D& pt);
      virtual void setPosition(ZPoint3D& pt);

      //overloads
    public:
      virtual DWORD  setStatus(const DWORD dwStatus)
        { return ZEntityBase::setStatus(dwStatus & ~STATUS_SELECTED); }
      virtual DWORD  changeStatus(const DWORD dwAdd, const DWORD dwRemove)
        { return ZEntityBase::changeStatus(dwAdd & ~STATUS_SELECTED, dwRemove & ~STATUS_SELECTED); }

      void init(scene::INode* pNode, IControlPoint* pControl, eControlVector type);
    protected:
      ZPtr<IControlPoint>     m_pControl;
      eControlVector          m_type;
  };

  //------------------------------
  //  The Spline-holder node class
  //------------------------------
  class CSplineNode : public ISplineNode,
                      public CTransformable,
                      public CBoundable,
                      public CVisualizeable,
                      public core::CNamed,
                      public IManipulateable,
                      public core::io::ISerializeable
  {
    public:
      INTERFACE_DECLARE_EMBEDMAP(CSplineNode)

    public:
      CSplineNode();
      virtual ~CSplineNode();
      void FinalConstruct();

    public:
    //-----------------------
    // INode
    //-----------------------
      virtual ZRESULT getParent(scene::INode** ppParent)
        { return DELIGATE(m_pNode)getParent(ppParent); }
      virtual ZRESULT getChildren(CNodeCol** ppChildren)
        { return DELIGATE(m_pNode)getChildren(ppChildren); }
      virtual ZRESULT setParent(scene::INode* pParent)
        { return DELIGATE(m_pNode)setParent(pParent); }
      virtual ZRESULT setChildren(CNodeCol* pChildren)
        { return DELIGATE(m_pNode)setChildren(pChildren); }
      virtual ZRESULT queryNodeOperation(eNodeOperation co, scene::INode* pNode, DWORD dwRes=0);
      virtual ZRESULT changed(DWORD dwBits);//specific behavior
      virtual ZRESULT clone(scene::INode** ppNode, bool bShareData=false);

    public:
    //-----------------------
    // ISplineNode
    //-----------------------
      virtual ZRESULT getSpline(scene::ISpline**  ppSpline);
      virtual ZRESULT setSpline(scene::ISpline*   pSpline);

    public:
    //-----------------------
    // CBoundable
    //-----------------------
      virtual ZRESULT updateBoundary();

    public:
      //overloads
      virtual DWORD  setStatus(const DWORD dwStatus);
      virtual DWORD  changeStatus(const DWORD dwAdd, const DWORD dwRemove);

    public:
    //-----------------------
    // IManipulateable
    //-----------------------
      virtual ZRESULT getManipulators(CManipulatorCol** ppPoints);
      virtual ZRESULT getManipulatorsByName(ZString strName, CManipulatorCol** ppPoints);

    public:
    //-----------------------
    // ISerializeable
    //-----------------------
      virtual ZRESULT saveDeclaration(core::io::IStream* pStream, services::IOpenSaveService* pSrv, LPCTSTR pszResolver=NULL);
      virtual ZRESULT saveData(core::io::IStream* pStream, DWORD dwID, services::IOpenSaveService* pSrv);
      virtual ZRESULT loadData(core::io::IStream* pStream, services::IOpenSaveService* pSrv, DWORD dwSize, DWORD dwVersion=0, bool bMerge=false);
      virtual ZRESULT getReCreationMode(core::io::eReCreationMode& mode);

    private:
    //-----------------------
    // implementation:
    //-----------------------
      virtual ZRESULT updateManipulators();

    private:
    //-----------------------
    // data members:
    //-----------------------
      ZPtr<CMiniNode>           m_pNode;
      ZPtr<scene::ISpline>      m_pSpline;
      ZPtr<CManipulatorCol>     m_pControllers;
      ZPtr<core::ui::controls::CUserDefinedOptions>   m_pUserOptions;
  };//class

};//scene

#endif //splinenode_class_h_included

