/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                   Copyright (c) 2002 Zanoza Software.                   *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x splines render technique
 *--------------------------------------------------------------------------
 * @histroy: 17.09.2003. Created
 * @histroy: 05.01.2004. Serialization added
 */
#ifndef zm_splines_render_technique_included
#define zm_splines_render_technique_included

#include <rend/IRenderTechnique.h>
#include <core/INamed.h>
#include <scene/ISplineNode.h>
#include <core/ui/IUIManager.h>
#include <helpers/layout.h>
#include <helpers/serializeable.h>

namespace rend
{

  //--------------------------------------------------------
  // the Splines rendering technique class
  //--------------------------------------------------------
  class CSplineRenderTechnique :  public rend::IRenderTechnique,
                                  public core::INamed,
                                  public core::io::ISerializeable,
                                  public core::CErrorInfo
  {
    public:
      INTERFACE_DECLAREMAP(CSplineRenderTechnique)

    public:
      CSplineRenderTechnique();
      virtual ~CSplineRenderTechnique();

    public:
    //-----------------------
    // rend::IRenderTechnique
    //-----------------------
      virtual ZRESULT validate(IDevice* pDevice, scene::INode* pNode);
      virtual ZRESULT nodeChange(scene::INode* pNode, DWORD dwBits);
      virtual ZRESULT renderNode(scene::INode* pNode, core::ui::IViewport* pViewport, IDevice* pDevice, ZMatrixSeq& matrixStack);
      virtual ZRESULT addRenderExtension(rend::IRenderExtension* pExt) {return ZRESULT_NOT_SUPPORTED; }
      virtual ZRESULT removeRenderExtension(rend::IRenderExtension* pExt) {return ZRESULT_NOT_SUPPORTED; }
      virtual ZRESULT getRenderExtensions(rend::CRenderExtensionCol** ppCol) {return ZRESULT_NOT_SUPPORTED; }

    public:
    //-----------------------
    // core::INamed
    //-----------------------
      virtual ZRESULT getName(ZString& str);
      virtual ZRESULT setName(ZString str);
      virtual ZRESULT getTitle(ZString& str);
      virtual ZRESULT setTitle(ZString str);
      virtual ZRESULT getDescription(ZStringSeq& seqDesc);
      virtual ZRESULT setDescription(ZStringSeq& seqDesc);

    public:
    //-----------------------
    // ISerializeable
    //-----------------------
      virtual ZRESULT saveDeclaration(core::io::IStream* pStream, services::IOpenSaveService* pSrv, LPCTSTR pszResolver=NULL);
      virtual ZRESULT saveData(core::io::IStream* pStream, DWORD dwID, services::IOpenSaveService* pSrv);
      virtual ZRESULT loadData(core::io::IStream* pStream, services::IOpenSaveService* pSrv, DWORD dwSize, DWORD dwVersion=0, bool bMerge=false);
      virtual ZRESULT getReCreationMode(core::io::eReCreationMode& mode);

    public:
    //-----------------------
    // implementation:
    //-----------------------
      ZRESULT         resizeBuffer(IDevice* pDevice, scene::ISpline* pSpline);
      ZRESULT         initSplineLines(scene::ISpline* pSpline, BYTE* pBuffer);
      ZRESULT         initSplineDots(scene::ISpline* pSpline, BYTE* pBuffer);
      ZRESULT         initSplineManipulators(scene::ISpline* pSpline, BYTE* pBuffer);

    private:
    //-----------------------
    // data members:
    //-----------------------
      ZString                     m_strName, m_strTitle;
      ZStringSeq                  m_seqDesc;
      ZStdPtr<scene::ISplineNode> m_pNode;
      // rendering stuff:
      core::eLevel                m_level;
      long                        m_nVertexBufferSize;  //size of allocated buffer in bytes
      long                        m_nVertexBufferCount; //amount of items written into buffer
      DWORD                       m_dwVertexStride;
      DWORD                       m_dwStatus;
      ZPtr<IVertexBuffer>         m_pVertices;          //lines, dots, manipulators...
      tRenderBlockSeq             m_seqBlocks;

      typedef struct td3dvertex
      {
        float x,y,z;
        DWORD     dwColor;
        td3dvertex& operator = (const ZPoint3D pt)
        {
          x = pt.x;
          y = pt.y;
          z = pt.z;
          return *this;
        }
        td3dvertex& operator + (const ZPoint3D pt)
        {
          x+= pt.x;
          y+= pt.y;
          z+= pt.z;
          return *this;
        }
      }td3dvertex;

      //
      // static datamembers
      //
      static ZStdPtr<core::ui::IGraphManager> m_pGraphManager;
      static std::map<core::ui::eColorResource, DWORD> g_sMapCol;

  };//class

};//rend

#endif //zm_splines_render_technique_included

